package net.sergeych.tools

/**
 * Multiplatform interface to perform a regular (not suspend) operation
 * protected by a platform mutex (where necessary). Get real implementation
 * with [ProtectedOp]
 */
interface ProtectedOpImplementation {
    /**
     * Call [f] iin mutually exclusive mode, it means that only one invocation
     * can be active at a time, all the rest are waiting until the current operation
     * will finish.
     */
    operator fun <T>invoke(f: ()->T): T
}


/**
 * Get the platform-depended implementation of a mutex-protected operation.
 */
expect fun ProtectedOp(): ProtectedOpImplementation